@@ -1,9 +1,9 @@
 #  Copyright (c) 2022 by Amplo.
 
 from amplo.automl.data_processing import DataProcessor
-from amplo.automl.data_sampling import DataSampler
 from amplo.automl.drift_detection import DriftDetector
 from amplo.automl.feature_processing import FeatureProcessor
+from amplo.automl.grid_search import OptunaGridSearch
 from amplo.automl.interval_analysis import IntervalAnalyser
 from amplo.automl.modelling import Modeller
 from amplo.automl.sequencing import Sequencer
