@@ -1,12 +1,13 @@
 #  Copyright (c) 2022 by Amplo.
+from __future__ import annotations
 
-import warnings
-from typing import Union
+from warnings import warn
 
 import numpy as np
 import pandas as pd
 from scipy import stats
 
+from amplo.base import BasePredictor, LoggingMixin
 from amplo.utils import hist_search
 
 
@@ -14,77 +15,91 @@ class DataDriftWarning(Warning):
     pass
 
 
-class DriftDetector:
+class DriftDetector(LoggingMixin):
     # todo add second order pdf fit
     # todo add subsequence drift detection
 
     def __init__(
         self,
-        num_cols: list = None,
-        cat_cols: list = None,
-        date_cols: list = None,
+        target: str,
         n_bins: int = 500,
         sigma: int = 3,
         with_pdf: bool = False,
+        verbose: int = 1,
     ):
+        """Detects data drift in streamed input data.
+
+        Supports only  numerical data as off now.
+        - Numerical data is check by the discrete distribution
+        - Categorical by new columns ()
+        - Dates by their recency
+
+        parameters
+        ----------
+        target : str
+        n_bins : int, default =  500
+            Number of bins to put the data in
+        sigma : int, default = 3
+        with_pdf : bool, default = False
+        verbose : int, default = 1
         """
-        Detects data drift in streamed input data.
-        Supports numerical, categorical and datetime variables.
-        Due to streamed, we don't check distributions, just bins.
-        Categorical simply checks whether it's not a new column
-        Datetime simply checks whether the date is recent
-        """
+        super().__init__(verbose=verbose)
         # Copy kwargs
-        self.num_cols = [] if num_cols is None else num_cols
-        self.cat_cols = [] if cat_cols is None else cat_cols
-        self.date_cols = [] if date_cols is None else date_cols
+        self.target = target
         self.n_bins = n_bins
         self.with_pdf = with_pdf
         self.sigma = sigma
 
         # Initialize
-        self.bins = {}
-        for key in self.cat_cols:
-            self.bins[key] = {}
-        for key in self.num_cols:
-            self.bins[key] = (None, None)
-        self.output_bins = (None, None)
-        self.distributions = {}
+        self.bins_ = {}
+        self.output_bins_ = (None, None)
+        self.distributions_ = {}
+        self.is_fitted_ = False
 
     def fit(self, data: pd.DataFrame) -> "DriftDetector":
         """
         Fits the class object
         """
         # Numerical
-        self._fit_bins(data)
-        self._fit_distributions(data)
+        numeric = data.select_dtypes(include=np.number)  # type: ignore
+        self._fit_bins(numeric)
+        self._fit_distributions(numeric)
 
         # Categorical
+        # TODO: check for new values
 
+        # Dates
+        # TODO: check for recency
+
+        self.is_fitted_ = True
         return self
 
-    def check(self, data: pd.DataFrame):
+    def check(self, data: pd.DataFrame | pd.Series):
         """
         Checks a new dataframe for distribution drift.
         """
         violations = []
 
-        violations.extend(self._check_bins(data))
-        violations.extend(self._check_distributions(data))
+        # Convert series
+        if isinstance(data, pd.Series):
+            data = pd.DataFrame(data.to_dict(), index=[0])
+
+        # Numerical
+        numeric = data.select_dtypes(include=np.number)  # type: ignore
+        violations.extend(self._check_bins(numeric))
+        violations.extend(self._check_distributions(numeric))
 
         return violations
 
-    def fit_output(self, model, data: pd.DataFrame):
+    def fit_output(self, model: BasePredictor, data: pd.DataFrame):
         """
         Additionally to detecting input drift, we should also detect output drift. When
         the distribution of predicted outcomes change, it's often a sign that some under
         laying dynamics are shifting.
         """
-        assert hasattr(model, "predict"), "Model does not have 'predict' attribute."
-
         # If it's a classifier and has predict_proba, we use that :)
         if hasattr(model, "predict_proba"):
-            prediction = model.predict_proba(data)[:, 1]
+            prediction = model.predict_proba(data)[:, 1]  # type: ignore
         else:
             prediction = model.predict(data)
 
@@ -94,22 +109,25 @@ class DriftDetector:
             bins=self.n_bins,
             range=(mi - (ma - mi) / 10, ma + (ma - mi) / 10),
         )
-        self.output_bins = (x.tolist(), y.tolist())
+        self.output_bins_ = (x.tolist(), y.tolist())
 
-    def check_output(self, model, data: pd.DataFrame, add: bool = False):
+    def check_output(self, model: BasePredictor, data: pd.DataFrame, add: bool = False):
         """
         Checks the predictions of a model.
         """
-        assert hasattr(model, "predict"), "Model does not have 'predict' attribute."
+        if not self.is_fitted_:
+            raise ValueError("Object not yet fitted.")
 
         # If it's a classifier and has predict_proba, we use that :)
         if hasattr(model, "predict_proba"):
-            prediction = model.predict_proba(data)[:, 1]
+            prediction = model.predict_proba(data)[:, 1]  # type: ignore
         else:
             prediction = model.predict(data)
 
-        # Check all predictions
-        x, y = self.output_bins
+        # Unpack predictions
+        x, y = self.output_bins_
+        assert isinstance(x, list) and isinstance(y, list)
+
         count_drifts = 0
         for value in prediction:
             ind = hist_search(x, value)
@@ -118,9 +136,7 @@ class DriftDetector:
                 count_drifts += 1
         if count_drifts > 0:
             severity = count_drifts / len(prediction) * 100
-            warnings.warn(
-                DataDriftWarning(f"Output drift detected! Severity: {severity:.2f}%")
-            )
+            warn(f"Output drift detected! Severity: {severity:.2f}%", DataDriftWarning)
 
         # Add new output
         if add:
@@ -133,9 +149,9 @@ class DriftDetector:
         Useful to save :)
         """
         return {
-            "bins": self.bins,
-            "output_bins": self.output_bins,
-            "distributions": self.distributions,
+            "bins": self.bins_,
+            "output_bins": self.output_bins_,
+            "distributions": self.distributions_,
         }
 
     def load_weights(self, weights):
@@ -154,9 +170,9 @@ class DriftDetector:
             - "distributions" (dict):
                 Fitted distributions for all numeric keys.
         """
-        self.bins = weights.get("bins", {})
-        self.output_bins = weights.get("output_bins", (None, None))
-        self.distributions = weights.get("distributions", {})
+        self.bins_ = weights.get("bins", {})
+        self.output_bins_ = weights.get("output_bins", (None, None))
+        self.distributions_ = weights.get("distributions", {})
         return self
 
     def _fit_bins(self, data: pd.DataFrame):
@@ -164,18 +180,14 @@ class DriftDetector:
         Fits a histogram on each numerical column.
         """
         # Fit numerical
-        for key in self.num_cols:
+        for key in data.keys():
             ma, mi = data[key].max(), data[key].min()
             y, x = np.histogram(
                 data[key],
                 bins=self.n_bins,
                 range=(mi - (ma - mi) / 10, ma + (ma - mi) / 10),
             )
-            self.bins[key] = (x.tolist(), y.tolist())
-
-        # Fit categorical
-        for key in self.cat_cols:
-            self.bins[key] = data[key].value_counts().to_dict()
+            self.bins_[key] = (x.tolist(), y.tolist())
 
     def _check_bins(self, data: pd.DataFrame, add: bool = False):
         """
@@ -183,9 +195,12 @@ class DriftDetector:
         """
         violations = []
 
-        for key in self.num_cols:
+        for key in data.keys():
             # Get bins
-            x, y = self.bins[key]
+            if key not in self.bins_:
+                violations.append(key)
+                continue
+            x, y = self.bins_[key]
 
             # Check bins
             if isinstance(data, pd.DataFrame):
@@ -206,10 +221,10 @@ class DriftDetector:
             # Add data
             if add:
                 y += np.histogram(data[key], bins=x)
-                self.bins[key] = (x, y)
+                self.bins_[key] = (x, y)
 
         if len(violations) > 0:
-            warnings.warn(
+            warn(
                 DataDriftWarning(
                     f"Drift detected! "
                     f"{len(violations)} features outside training bins: {violations}"
@@ -228,8 +243,8 @@ class DriftDetector:
             fitted = []
 
             # Iterate through numerical columns
-            for key in self.num_cols:
-                y, x = np.histogram(data[key], normed=True)
+            for key in data.keys():
+                y, x = np.histogram(data[key], normed=True)  # type: ignore
                 x = (x + np.roll(x, -1))[:-1] / 2.0  # Get bin means
 
                 # Iterate through distributions
@@ -253,7 +268,7 @@ class DriftDetector:
                     )
 
                 # Select lowest
-                self.distributions[key] = fitted[np.argmin(distances)]
+                self.distributions_[key] = fitted[np.argmin(distances)]
 
     def _check_distributions(self, data: pd.DataFrame) -> list:
         """
@@ -264,9 +279,9 @@ class DriftDetector:
 
         if self.with_pdf:
             # Check all numerical columns
-            for key in self.num_cols:
-                dist = getattr(stats, self.distributions[key]["distribution"])
-                params = self.distributions[key]["params"]
+            for key in data.keys():
+                dist = getattr(stats, self.distributions_[key]["distribution"])
+                params = self.distributions_[key]["params"]
                 probabilities = dist.pdf(
                     data[key].values, loc=params[-2], scale=params[-1], *params[:-2]
                 )
@@ -276,7 +291,7 @@ class DriftDetector:
                     continue
 
             if len(violations) > 0:
-                warnings.warn(
+                warn(
                     f"Drift detected! {len(violations)} features outside training bins:"
                     f" {violations}",
                     DataDriftWarning,
@@ -284,9 +299,7 @@ class DriftDetector:
 
         return violations
 
-    def add_output_bins(
-        self, old_bins: tuple, prediction: Union[np.ndarray, pd.Series]
-    ):
+    def add_output_bins(self, old_bins: tuple, prediction: np.ndarray | pd.Series):
         """
         Just a utility, adds new data to an old distribution.
         """
@@ -298,31 +311,3 @@ class DriftDetector:
             y, x = np.histogram(prediction, bins=self.n_bins)
             x, y = x.tolist(), y.tolist()
         return x, y
-
-    def add_bins(self, bins: dict, data: pd.DataFrame):
-        """
-        Just a utility, adds new data to an old distribution.
-        """
-        # Add numerical bins
-        for key in self.num_cols:
-            if key in data:
-                if key in bins:
-                    x, y = bins[key]
-                elif key in self.bins:
-                    x, _ = self.bins[key]
-                    y = [0 for i in range(len(x) - 1)]
-                else:
-                    raise ValueError(f"Drift Detector - Adding unfitted feature. {key}")
-                yn = np.histogram(data[key], bins=x)[0].tolist()
-                y = [y[i] + yn[i] for i in range(len(y))]
-                bins[key] = (x, y)
-
-        # Add categorical
-        for key in self.cat_cols:
-            if key in data:
-                counts = data[key].value_counts().to_dict()
-                bins[key] = {
-                    k: counts.get(k, 0) + bins.get(key, {}).get(k, 0)
-                    for k in counts.keys() | bins.get(key, {}).keys()
-                }
-        return bins
