@@ -11,15 +11,17 @@ References
 The ML test score: A rubric for ML production readiness and technical debt
 reduction. 1123-1132. 10.1109/BigData.2017.8258038.
 """
+from __future__ import annotations
 
 import json
 
 import numpy as np
 import pandas as pd
 from cleanlab.filter import find_label_issues
+from sklearn.ensemble import RandomForestClassifier
 from sklearn.model_selection import cross_val_predict
 
-from amplo.observation._base import PipelineObserver, _report_obs
+from amplo.observation._base import BaseObserver, _report_obs
 from amplo.utils.logging import get_root_logger
 from amplo.utils.metrics import levenshtein_distance
 
@@ -29,7 +31,7 @@ __all__ = ["DataObserver"]
 logger = get_root_logger().getChild("DataObserver")
 
 
-class DataObserver(PipelineObserver):
+class DataObserver(BaseObserver):
     """
     Data observer before pushing to production.
 
@@ -46,15 +48,22 @@ class DataObserver(PipelineObserver):
 
     _obs_type = "data_observation"
 
-    def observe(self):
-        self.check_monotonic_columns()
-        self.check_minority_sensitivity()
-        self.check_extreme_values()
-        self.check_categorical_mismatch()
-        self.check_label_issues()
+    def observe(
+        self,
+        data: pd.DataFrame,
+        mode: str,
+        target: str,
+        dummies=None,
+    ) -> list[dict[str, str | bool]]:
+        self.check_monotonic_columns(data)
+        self.check_minority_sensitivity(data, target)
+        self.check_extreme_values(data)
+        self.check_categorical_mismatch(dummies)
+        self.check_label_issues(data, mode, target)
+        return self.observations
 
     @_report_obs
-    def check_monotonic_columns(self):
+    def check_monotonic_columns(self, data: pd.DataFrame):
         """
         Checks whether any column is monotonically in- or decreasing.
 
@@ -68,8 +77,8 @@ class DataObserver(PipelineObserver):
             A brief description of the observation and its results.
         """
         logger.info("Checking data for monotonic columns (often indices).")
-        x_data = pd.DataFrame(self.x)
-        numeric_data = x_data.select_dtypes(include=np.number)
+        numerics = ["int16", "int32", "int64", "float16", "float32", "float64"]
+        numeric_data = data.select_dtypes(include=numerics)
 
         # Make multi-indexed if not already
         if len(numeric_data.index.names) == 1:
@@ -78,7 +87,7 @@ class DataObserver(PipelineObserver):
         # Sort index from the innermost to outermost index level since it is shuffled
         # in classification.
         for axis in list(range(len(numeric_data.index.names)))[::-1]:
-            numeric_data.sort_index(axis=axis, inplace=True)
+            numeric_data.sort_index(axis=axis, inplace=True)  # type: ignore
 
         monotonic_columns = []
         for col in numeric_data.columns:
@@ -104,7 +113,7 @@ class DataObserver(PipelineObserver):
         return status_ok, message
 
     @_report_obs
-    def check_minority_sensitivity(self):
+    def check_minority_sensitivity(self, data: pd.DataFrame, target: str):
         """
         Checks whether the data has sensitivities towards minority classes.
 
@@ -130,33 +139,34 @@ class DataObserver(PipelineObserver):
         """
         logger.info("Checking data for minority sensitive columns.")
         minority_sensitive = []
+        y = data[target]
 
-        for key in self.x.keys():
-            if not pd.api.types.is_numeric_dtype(self.x[key]):
+        for key in data.keys():
+            if not pd.api.types.is_numeric_dtype(data[key]):
                 # Todo implement for categorical columns
                 continue
 
             # Make bins
-            counts, edges = np.histogram(self.x[key].fillna(0), bins=10)
+            counts, edges = np.histogram(data[key].fillna(0), bins=10)
 
             # Check if a minority group is present
             minority_size = min([c for c in counts if c != 0])
-            if minority_size > len(self.x) // 50:
+            if minority_size > len(data) // 50:
                 continue
 
             # If present, check the labels
-            bin_indices = np.digitize(self.x[key], bins=edges)
+            bin_indices = np.digitize(data[key], bins=edges)
             for bin_ind in np.where(counts == minority_size)[0]:
                 minority_indices = np.where(bin_indices == bin_ind + 1)[0]
 
                 # No minority if spread across labels
-                if self.y.iloc[minority_indices].nunique() != 1:
+                if y.iloc[minority_indices].nunique() != 1:
                     continue
 
                 # Not sensitive if only a fraction of the label
                 if (
                     len(minority_indices)
-                    > (self.y == self.y.iloc[minority_indices[0]]).sum() // 5
+                    > (y == y.iloc[minority_indices[0]]).sum() // 5
                 ):
                     minority_sensitive.append(key)
 
@@ -169,7 +179,7 @@ class DataObserver(PipelineObserver):
         return status_ok, message
 
     @_report_obs
-    def check_extreme_values(self):
+    def check_extreme_values(self, data: pd.DataFrame):
         """
         Checks whether extreme values are present.
 
@@ -183,12 +193,12 @@ class DataObserver(PipelineObserver):
         logger.info("Checking data for extreme values.")
         extreme_values = []
 
-        for key in self.x.keys():
-            if not pd.api.types.is_numeric_dtype(self.x[key]):
+        for key in data.keys():
+            if not pd.api.types.is_numeric_dtype(data[key]):
                 # Todo implement for categorical columns
                 continue
 
-            if self.x[key].abs().max() > 1000:
+            if data[key].abs().max() > 1000:
                 extreme_values.append(key)
 
         status_ok = not extreme_values
@@ -199,7 +209,7 @@ class DataObserver(PipelineObserver):
         return status_ok, message
 
     @_report_obs
-    def check_categorical_mismatch(self):
+    def check_categorical_mismatch(self, dummies=None):
         """
         Checks whether categorical variables are mismatched
 
@@ -215,9 +225,11 @@ class DataObserver(PipelineObserver):
         """
         logger.info("Checking data for categorical mismatches.")
         categorical_mismatches = []
+        if not dummies:
+            dummies = {}
 
         # Get dummy information from pipeline
-        for feature, variants in self._pipe.data_processor.dummies.items():
+        for feature, variants in dummies.items():
 
             # Check if one is similar
             for i, variant_x in enumerate(variants):
@@ -227,8 +239,8 @@ class DataObserver(PipelineObserver):
                         continue
 
                     # Remove feature from variants
-                    variant_x = variant_x[len(feature) + 1 :]
-                    variant_y = variant_y[len(feature) + 1 :]
+                    variant_x = variant_x[len(feature) + 1 :].lower()
+                    variant_y = variant_y[len(feature) + 1 :].lower()
 
                     # Get distance (normalized)
                     distance = levenshtein_distance(variant_x, variant_y) / max(
@@ -247,24 +259,34 @@ class DataObserver(PipelineObserver):
         return status_ok, message
 
     @_report_obs
-    def check_label_issues(self):
+    def check_label_issues(self, data: pd.DataFrame, mode: str, target: str):
         """
         Checks for label issues (classification only). Uses cleanlabs.
         """
         # Only classification
-        if self._pipe.mode != "classification":
+        if mode != "classification":
             return True, "All good"
 
+        logger.info("Checking data for label issues.")
+
+        y = data[target]
+        x = data.drop(target, axis=1)
+
         # Get predicted labels
         predicted_probabilities = cross_val_predict(
-            self.model, self.x, self.y, cv=3, method="predict_proba"
+            RandomForestClassifier(),
+            x,
+            y,
+            cv=3,
+            method="predict_proba",
         )
 
         # Analyse with cleanlabs
         ranked_label_issues = find_label_issues(
-            labels=self.y,
+            labels=y.values.astype("int"),
             pred_probs=predicted_probabilities,
             return_indices_ranked_by="self_confidence",
+            n_jobs=1,
         ).tolist()
 
         # Return
