@@ -7,7 +7,7 @@ import os
 import re
 from pathlib import Path
 from time import time
-from typing import TYPE_CHECKING, Iterable, cast
+from typing import TYPE_CHECKING, Iterable
 from warnings import warn
 
 import numpy as np
@@ -28,7 +28,6 @@ __all__ = [
     "NpEncoder",
     "read_pandas",
     "get_file_metadata",
-    "merge_folders",
     "merge_logs",
 ]
 
@@ -52,6 +51,10 @@ class NpEncoder(json.JSONEncoder):
             return float(obj)
         if isinstance(obj, np.ndarray):
             return obj.tolist()
+        if isinstance(obj, pd.Series):
+            return obj.to_list()
+        if isinstance(obj, pd.DataFrame):
+            obj.to_json()
         return super(NpEncoder, self).default(obj)
 
 
@@ -140,11 +143,64 @@ def get_file_metadata(file_path: str | Path) -> dict[str, str | float]:
     }
 
 
+def _get_api_clients(
+    azure: tuple[str, str] | bool = False,
+    platform: tuple[str, str] | bool | None = None,
+):
+    """
+    Gathers the api clients for merge_logs
+    """
+    from amplo.api.platform import AmploPlatformAPI
+
+    if not azure:
+        blob_api = None
+    else:
+        azure = azure if not isinstance(azure, bool) else tuple()
+        blob_api = AzureBlobDataAPI.from_os_env(*azure)
+
+    # Mirror azure parameter when platform is not set
+    if platform is None:
+        platform = bool(azure)
+    # Get amplo platform client
+    if not platform:
+        platform_api = None
+    else:
+        platform = platform if not isinstance(platform, bool) else tuple()
+        platform_api = AmploPlatformAPI.from_os_env(*platform)
+    return blob_api, platform_api
+
+
+def _get_folders(
+    parent_folder: str | Path,
+    blob_api: AzureBlobDataAPI | None = None,
+    more_folders: list[str | Path] | None = None,
+) -> list[Path]:
+    """
+    Lists folders for merge_logs
+    """
+    if not blob_api:
+        if not Path(parent_folder).exists():
+            raise ValueError(f"{parent_folder} directory does not exist.")
+        folders = [
+            folder for folder in Path(parent_folder).iterdir() if folder.is_dir()
+        ]
+    else:
+        folders = blob_api.ls_folders(parent_folder)
+
+    # Add more_folders
+    if more_folders:
+        folders += more_folders
+
+    # return [Path(f) for f in folders]
+    return [Path(f) for i, f in enumerate(folders) if i < 3]
+
+
 def _read_files_in_folders(
     folders: Iterable[str | Path],
+    target: str,
     blob_api: AzureBlobDataAPI | None = None,
     logger: Logger | None = None,
-) -> tuple[list[str], list[pd.DataFrame], list[dict[str, str | float]]]:
+) -> tuple[list[str], pd.DataFrame, dict[str, dict]]:
     """
     Use pandas to read all non-hidden and non-empty files into a DataFrame.
 
@@ -152,6 +208,8 @@ def _read_files_in_folders(
     ----------
     folders : iterable of (str or Path)
         Directory names.
+    target : str
+        Target column & directory name
     blob_api : AzureBlobDataAPI or None, optional, default: None
         If None, tries to read data from local folder, else from Azure
     logger : Logger or None, optional, default: None
@@ -159,10 +217,10 @@ def _read_files_in_folders(
 
     Returns
     -------
+    filenames : list of str
     data : pd.DataFrame
         All files of the folders merged into one multi-indexed DataFrame.
     metadata : list of dict of {str : str or float}
-        Metadata of data.
 
     Warnings
     --------
@@ -171,11 +229,10 @@ def _read_files_in_folders(
     """
 
     # Map folders to pathlib.Path object
-    folders = list(map(Path, folders))
-    folders = cast(list[Path], folders)  # type hint
+    folders = [Path(f) for f in folders]
 
     # Initialize
-    file_names, data, metadata = [], [], []
+    file_names, data, metadata = [], [], {}
     last_time_logged = time()
     for folder_count, folder in enumerate(sorted(folders)):
 
@@ -206,32 +263,53 @@ def _read_files_in_folders(
             continue
 
         # Read files
-        for file in sorted(files):
+        for file_ in sorted(files):
 
             # read_pandas() may raise an EmptyDataError when the file has no content.
             # The try...except catches such errors and warns the user instead.
             try:
                 if blob_api:
-                    datum = blob_api.read_pandas(file)
-                    metadatum = blob_api.get_metadata(file)
+                    datum = blob_api.read_pandas(file_)
+                    metadatum = blob_api.get_metadata(file_)
                 else:
-                    datum = read_pandas(file)
-                    metadatum = get_file_metadata(file)
+                    datum = read_pandas(file_)
+                    metadatum = get_file_metadata(file_)
             except pd.errors.EmptyDataError:
-                warn(f"Empty file detected and thus skipped: '{file}'")
+                warn(f"Empty file detected and thus skipped: '{file_}'")
                 continue
 
+            # Convert to dataframe
             if isinstance(datum, pd.Series):
                 datum = datum.to_frame()
 
-            file_names.append(str(file))
+            # Set multi-index
+            datum = datum.set_index(
+                pd.MultiIndex.from_product(
+                    [[str(file_)], datum.index.values], names=["log", "index"]
+                )
+            )
+
+            # Add target
+            datum[target] = (folder.name == target) * 1
+
+            # Append
+            file_names.append(str(file_))
             data.append(datum)
-            metadata.append(metadatum)
+            metadata[str(file_)] = metadatum
 
         if logger and time() - last_time_logged > 90:
             last_time_logged = time()
             logger.info(f".. progress: {folder_count / len(folders) * 100:.1f} %")
 
+    # Concatenate data
+    data = pd.concat(data, axis=0)
+
+    # Validate data
+    if target not in data:
+        raise ValueError("Target not in data.")
+    if data[target].nunique() != 2:
+        raise ValueError(f"Number of unique labels is {data[target].nunique()} != 2.")
+
     return file_names, data, metadata
 
 
@@ -239,7 +317,7 @@ def _map_datalogs_to_file_names(
     file_names: list[str],
     platform_api: AmploPlatformAPI | None = None,
     logger: Logger | None = None,
-) -> list[dict]:
+) -> dict:
     """
     Get datalogs for every filename.
 
@@ -259,7 +337,7 @@ def _map_datalogs_to_file_names(
     """
 
     if not platform_api:
-        return []
+        return {}
 
     # It is assumed that the 6th and 5th path position of the (first) filename contains
     # the team and machine name, respectively, if you count from right to left.
@@ -272,10 +350,10 @@ def _map_datalogs_to_file_names(
         team, machine = file_names[0].split("/")[-6:-4]
     except IndexError:
         warn(f"Got an empty list of file names")
-        return []
+        return {}
 
     # Get datalog for each filename
-    datalogs = []
+    datalogs = {}
     last_time_logged = time()
     for file_count, fname in enumerate(file_names):
         try:
@@ -284,7 +362,7 @@ def _map_datalogs_to_file_names(
             # No matching datalog found. Do still append it to preserve the order.
             datalog = {}
 
-        datalogs.append(datalog)
+        datalogs[fname] = datalog
 
         if logger and time() - last_time_logged > 90:
             last_time_logged = time()
@@ -293,11 +371,7 @@ def _map_datalogs_to_file_names(
     return datalogs
 
 
-def _mask_intervals(
-    datalogs: list[dict],
-    data: list[pd.DataFrame],
-    metadata: list[dict[str, str | float]],
-) -> tuple[list[pd.DataFrame], list[dict[str, str | float]]]:
+def _mask_intervals(datalogs: dict, data: pd.DataFrame) -> pd.DataFrame:
     """
     Masks the data with the intervals given by the datalogs.
 
@@ -305,17 +379,13 @@ def _mask_intervals(
     ----------
     datalogs : list of dict
         Datalogs dictionary that should contain the keys 'selected' and 'datetime_col'.
-    data : list of pd.DataFrame
+    data : pd.DataFrame
         Data for splitting.
-    metadata : list of dict of {str : str or float}
-        Metadata of data. For each data split it will be duplicated.
 
     Returns
     -------
-    data_out : list of pd.DataFrame
+    data_out : pd.DataFrame
         Selected data.
-    metadata_out : list of dict of {str : str or float}
-        Same metadata but duplicated for every split.
 
     Warnings
     --------
@@ -323,190 +393,46 @@ def _mask_intervals(
         When no valid match for the start or stop time of the data interval was found,
         i.e. when the time difference is more than 1 second.
     """
-
-    # Initialize
-    data_out = []
-    metadata_out = []
-
-    for datalog, datum, metadatum in zip(datalogs, data, metadata):
+    for filename in data.index.get_level_values("log").unique():
         # Get intervals and timestamp column from datalog
+        datalog = datalogs.get(filename, {})
         intervals = datalog.get("selected", [])
         ts_col = datalog.get("datetime_col", "")
 
-        # If no interval is to be selected, append the whole datum
+        # Validate
         if not intervals or not ts_col:
-            data_out.append(datum)
-            metadata_out.append(metadatum)
-
-        # Prevent a KeyError when ts_col column is not present
-        elif ts_col not in datum.columns:
+            continue
+        elif ts_col not in data:
             warn(f"Cannot select intervals as the column '{ts_col}' is not present.")
-            data_out.append(datum)
-            metadata_out.append(metadatum)
+            continue
 
-        # Else, select and append each interval of the datum
-        else:
-            # Extract unix timestamps
-            ts = datum[ts_col]
-            if not pd.api.types.is_datetime64_any_dtype(ts):
-                ts = pd.to_datetime(ts)
-            ts = ts.astype(int) / 10**9  # convert to unix format
-
-            # Extract intervals
-            for interval in intervals:
-                # Find closest timestamps
-                ts_first, ts_last = interval
-                first = (ts - ts_first).abs().argmin()
-                last = (ts - ts_last).abs().argmin()
-
-                # Ignore when time difference is too large
-                if (
-                    abs(ts.iloc[first] - ts_first) > 1
-                    or abs(ts.iloc[last] - ts_last) > 1
-                ):
-                    warn(
-                        f"Could not find a timestamp close enough to {ts_first=} or "
-                        f"{ts_last=}. Using the whole datum for the file "
-                        f"'{metadatum.get('file_name', 'unknown')}' instead."
-                    )
-                    data_out.append(datum)
-                    metadata_out.append(metadatum)
-                    continue
-
-                # Select interval
-                data_out.append(datum.iloc[first : last + 1])
-                metadata_out.append(metadatum)
-
-    return data_out, metadata_out
-
-
-def _make_multiindex(
-    data: list[pd.DataFrame],
-    metadata: list[dict[str, str | float]],
-    target_col: str = "labels",
-) -> tuple[pd.DataFrame, dict[int, dict[str, str | float]]]:
-    """
-    Merge list of dataframes into one multiindexed one.
-
-    Parameters
-    ----------
-    data : list of pd.DataFrame
-        Data to be merged.
-    metadata : list of dict of {str : str or float}
-        Metadata of data.
-    target_col : str, optional
-        Target column name. Values are depicted by the folder name, by default "labels"
-
-    Returns
-    -------
-    data : pd.DataFrame
-        All files of the folders merged into one multi-indexed DataFrame.
-    metadata : dict of {int : dict of {str : str or float}}
-        Metadata of merged data.
-        The first index level values of the data correspond to the keys in the metadata.
-
-    Raises
-    ------
-    ValueError
-        When any file already has a column named after target_col and its values are not
-        equal to the folder name.
-    """
-
-    # Initialize
-    index_count = 0
-    data_out = []
-    metadata_out = {}
-
-    for datum, metadatum in zip(data, metadata):
-
-        # Get file and folder name
-        full_path = str(metadatum["full_path"])
-        folder_name = Path(full_path).parent.name
-
-        # Set label column
-        if target_col in datum.columns and any(datum[target_col] != folder_name):
-            raise ValueError(
-                f"The column '{target_col}' already exists in the file '{full_path}'."
+        # Convert ts_col
+        if not pd.api.types.is_numeric_dtype(data[ts_col]):
+            data[ts_col] = (
+                pd.to_datetime(data[ts_col], errors="coerce").astype(int) / 10**9
             )
-        else:
-            datum[target_col] = folder_name
 
-        # Set multiindex
-        index = pd.MultiIndex.from_product(
-            [[index_count], datum.index.values], names=["log", "index"]
-        )
-        datum.set_index(index, inplace=True)
+        # Extract intervals
+        drop_mask = True
+        for interval in intervals:
+            ts_first, ts_last = interval
+            drop_mask = (
+                (data[ts_col] < ts_first) | (data[ts_col] > ts_last)
+            ) & drop_mask
+        if not drop_mask.loc[filename].any():
+            continue
+        data = data.drop(data.loc[(filename, drop_mask), :].index)
 
-        # Add data and metadata, and increment
-        data_out.append(datum)
-        metadata_out[index_count] = metadatum
-        index_count += 1
-
-    # Finish: concatenate data
-    return pd.concat(data_out), metadata_out
-
-
-def merge_folders(
-    folders: Iterable[str | Path],
-    target_col: str = "labels",
-    blob_api: AzureBlobDataAPI | None = None,
-    platform_api: AmploPlatformAPI | None = None,
-    logging: bool = False,
-) -> tuple[pd.DataFrame, dict[int, dict[str, str | float]]]:
-    """
-    Combine log files from given directories into a multiindexed DataFrame.
-
-    Parameters
-    ----------
-    folders : iterable of (str or Path)
-        Directory paths to read data from.
-    target_col : str, optional, default: "labels"
-        Target column name. Values are depicted by the folder name
-    blob_api : AzureBlobDataAPI or None, optional, default: None
-        Azure api when not reading from local
-    platform_api : AmploPlatformAPI or None, optional default: None
-        Platform api for selecting intervals
-    logging : bool, default: False
-        Whether to show logging info.
-
-    Returns
-    -------
-    data : pd.DataFrame
-        All files of the folders merged into one multi-indexed DataFrame.
-    metadata : dict of {int : dict of {str : str or float}}
-        Metadata of merged data.
-        The first index level values of the data correspond to the keys in the metadata.
-    """
-
-    logger = get_root_logger() if logging else None
-
-    # Pandas read files
-    logger.info("Reading files") if logger else None
-    fnames, data, metadata = _read_files_in_folders(folders, blob_api, logger)
-
-    # Select intervals when datalogs are available
-    logger.info("Reading datalogs from platform") if logger else None
-    datalogs = _map_datalogs_to_file_names(fnames, platform_api, logger)
-    if datalogs:
-        logger.info("Masking intervals from datalogs") if logger else None
-        data, metadata = _mask_intervals(datalogs, data, metadata)
-
-    # Concatenate data and make it multiindexed
-    logger.info("Making multiindexed data") if logger else None
-    data, metadata = _make_multiindex(data, metadata, target_col)
-
-    return data, metadata
+    return data
 
 
 def merge_logs(
-    parent_folder: str | Path,
-    target_col: str = "labels",
-    *,
+    parent: str | Path,
+    target: str,
     more_folders: list[str | Path] | None = None,
     azure: tuple[str, str] | bool = False,
     platform: tuple[str, str] | bool | None = None,
-    logging: bool = False,
-) -> tuple[pd.DataFrame, dict[int, dict[str, str | float]]]:
+) -> tuple[pd.DataFrame, dict[str, dict]]:
     """
     Combine log files of all subdirectories into a multi-indexed DataFrame.
 
@@ -534,8 +460,8 @@ def merge_logs(
     ----------
     parent_folder : str or Path
         Directory that contains subdirectories with tabular data files.
-    target_col : str
-        Target column name. Values are depicted by the folder name.
+    target : str
+        The target folder.
     more_folders : list of str or Path, optional
         Additional folder names with tabular data files to append.
     azure : (str, str) or bool, default: False
@@ -549,8 +475,8 @@ def merge_logs(
         If False, no datalogs information will be used.
         If True, the AmploPlatformAPI is initialized with default OS env variables.
         Otherwise, it will use the tuple to initialize the api.
-    logging : bool, default: False
-        Whether to show logging info.
+    verbose : int, default = 1
+
 
     Returns
     -------
@@ -560,41 +486,34 @@ def merge_logs(
     metadata : dict of {int : dict of {str : str or float}}
         Metadata of merged data.
     """
-
-    from amplo.api.platform import AmploPlatformAPI
     from amplo.utils import check_dtypes
 
-    check_dtypes("parent_folder", parent_folder, (str, Path))
-    check_dtypes("target_col", target_col, str)
-    check_dtypes("more_folders", more_folders, (type(None), list))
+    logger = get_root_logger()
+    check_dtypes(
+        ("parent_folder", parent, (str, Path)),
+        ("target_col", target, str),
+        ("more_folders", more_folders, (type(None), list)),
+    )
 
     # Get azure blob client
-    if not azure:
-        blob_api = None
-    else:
-        azure = azure if not isinstance(azure, bool) else tuple()
-        blob_api = AzureBlobDataAPI.from_os_env(*azure)
-
-    # Mirror azure parameter when platform is not set
-    if platform is None:
-        platform = bool(azure)
-    # Get amplo platform client
-    if not platform:
-        platform_api = None
-    else:
-        platform = platform if not isinstance(platform, bool) else tuple()
-        platform_api = AmploPlatformAPI.from_os_env(*platform)
+    blob_api, platform_api = _get_api_clients(azure, platform)
 
     # Get child folders
-    if not blob_api:
-        folders = [
-            folder for folder in Path(parent_folder).iterdir() if folder.is_dir()
-        ]
-    else:
-        folders = blob_api.ls_folders(parent_folder)
+    folders = _get_folders(parent, blob_api)
+    if target not in [f.name for f in folders]:
+        raise ValueError(f"Target {target} not present in folders.")
+    logger.info(f"Found {len(folders)} folders.")
 
-    # Add more_folders
-    if more_folders:
-        folders += more_folders
+    # Pandas read files
+    fnames, data, metadata = _read_files_in_folders(folders, target, blob_api, logger)
+    logger.info(f"Found {len(fnames)} files.")
 
-    return merge_folders(folders, target_col, blob_api, platform_api, logging)
+    # Masking data
+    if platform_api:
+        logger.info("Reading datalogs from platform")
+        datalogs = _map_datalogs_to_file_names(fnames, platform_api, logger)
+        if datalogs:
+            logger.info("Masking intervals from datalogs")
+            data = _mask_intervals(datalogs, data)
+
+    return data, metadata
