@@ -2,6 +2,7 @@
 
 import json
 
+import pandas as pd
 import pytest
 
 from amplo.automl.feature_processing.nop_feature_extractor import NopFeatureExtractor
@@ -9,26 +10,27 @@ from amplo.automl.feature_processing.nop_feature_extractor import NopFeatureExtr
 
 class TestNopFeatureExtractor:
     @pytest.mark.parametrize("mode", ["classification", "regression"])
-    def test_all(self, mode, make_x_y):
-        x, y = make_x_y
-        fe = NopFeatureExtractor()
+    def test_all(self, mode, make_data):
+        data = make_data
+        fe = NopFeatureExtractor(target="target")
 
         # Test fit and fit_transform
-        out1 = fe.fit_transform(x, y)
-        out2 = fe.transform(x)
+        out1 = fe.fit_transform(data)
+        out2 = fe.transform(data)
         assert all(out1 == out2), "`fit_transform` and `transform` don't match."
 
         # Test features_
-        assert set(fe.features_) == set(x), "Not all / too many features accepted."
+        features = set(data) - {"target"}
+        assert set(fe.features_) == features, "Not all / too many features accepted."
 
         # Test settings
         new_fe = NopFeatureExtractor().load_settings(fe.get_settings())
-        new_out = new_fe.transform(x)
-        assert set(new_fe.features_) == set(x), "Features not correctly restored."
+        new_out = new_fe.transform(data)
+        assert set(new_fe.features_) == features, "Features not correctly restored."
         assert all(out1 == new_out), "Transformation not correct."
 
         # Test JSON serializable
         settings = json.loads(json.dumps(fe.get_settings()))
         new_fe = NopFeatureExtractor().load_settings(settings)
         assert fe.get_settings() == new_fe.get_settings()
-        assert all(fe.transform(x) == new_fe.transform(x))
+        assert all(fe.transform(data) == new_fe.transform(data))
