@@ -1,29 +1,30 @@
-import pytest
+#  Copyright (c) 2022 by Amplo.
+
 import os
-import joblib
 from typing import Union
 
+import joblib
 import numpy as np
 import pandas as pd
-
-from sklearn.datasets import make_classification, make_regression
+import pytest
 from sklearn.base import clone
-from sklearn.metrics import SCORERS
+from sklearn.datasets import make_classification, make_regression
+from sklearn.metrics import get_scorer
 
-from Amplo.Classifiers import (
+from amplo.base.exceptions import NotFittedError
+from amplo.classification import (
     CatBoostClassifier,
     LGBMClassifier,
-    XGBClassifier,
     StackingClassifier,
+    XGBClassifier,
 )
-from Amplo.Regressors import (
+from amplo.regression import (
     CatBoostRegressor,
     LGBMRegressor,
-    XGBRegressor,
     StackingRegressor,
+    XGBRegressor,
 )
 
-
 Model = Union[
     CatBoostClassifier,
     LGBMClassifier,
@@ -63,9 +64,9 @@ def setup_class(request):
 
     if "Stacking" in model.__name__:
         if is_classification:
-            model_params = {"CatBoostClassifier": {"depth": 10}}
+            model_params = {"DecisionTreeClassifier__max_depth": 10}
         else:
-            model_params = {"CatBoostRegressor": {"depth": 10}}
+            model_params = {"DecisionTreeRegressor__max_depth": 10}
     else:
         model_params = {"max_depth": 10}
 
@@ -78,7 +79,7 @@ def setup_class(request):
 
 
 @pytest.mark.usefixtures("setup_class")
-class TestClassifier:
+class TestModel:
     @pytest.fixture(autouse=True)
     def setup(self):
         # Initialize model
@@ -97,11 +98,11 @@ class TestClassifier:
     def test_fit_numpy(self):
         self.model.fit(self.x.to_numpy(), self.y.to_numpy())
 
-    def test_trained_attr(self):
-        assert hasattr(self.model, "trained")
-        assert self.model.trained is False
+    def test_is_fitted(self):
+        with pytest.raises(NotFittedError):
+            self.model.check_is_fitted()
         self.model.fit(self.x, self.y)
-        assert self.model.trained is True
+        self.model.check_is_fitted()
 
     def test_predict(self):
         self.model.fit(self.x, self.y)
@@ -144,7 +145,7 @@ class TestClassifier:
             scorers = ["neg_mean_squared_error", "neg_mean_absolute_error", "r2"]
 
         for name in scorers:
-            SCORERS[name](self.model, self.x, self.y)
+            get_scorer(name)(self.model, self.x, self.y)
 
     def test_binary(self):
         if self.is_classification:
@@ -152,9 +153,9 @@ class TestClassifier:
             self.model.fit(x, y)
             self.model.predict(x)
             self.model.predict_proba(x)
-            SCORERS["neg_log_loss"](self.model, x, y)
-            SCORERS["accuracy"](self.model, x, y)
-            SCORERS["f1_micro"](self.model, x, y)
+            get_scorer("neg_log_loss")(self.model, x, y)
+            get_scorer("accuracy")(self.model, x, y)
+            get_scorer("f1_micro")(self.model, x, y)
 
     def test_pickleable(self):
         x, y = make_classification()
